lists.openwall.net   lists  /  announce  owl-users  owl-dev  john-users  john-dev  passwdqc-users  yescrypt  popa3d-users  /  oss-security  kernel-hardening  musl  sabotage  tlsify  passwords  /  crypt-dev  xvendor  /  Bugtraq  Full-Disclosure  linux-kernel  linux-netdev  linux-ext4  linux-hardening  linux-cve-announce  PHC 
Open Source and information security mailing list archives
 
Hash Suite: Windows password security audit tool. GUI, reports in PDF.
[<prev] [next>] [<thread-prev] [thread-next>] [day] [month] [year] [list]
Date:	Sat, 14 Jul 2007 19:52:43 -0400
From:	Mathieu Desnoyers <mathieu.desnoyers@...ymtl.ca>
To:	akpm@...ux-foundation.org, linux-kernel@...r.kernel.org
Cc:	Christoph Hellwig <hch@...radead.org>
Subject: [PATCH] Immediate Value - Architecture Independent Code Deferred Sync

Linux Kernel Markers - Architecture Independent Code Deferred Sync

Upon marker probe_unregister, we delay call to synchronize_sched() to
accelerate mass unregistration (only when there is no more reference to a
give module do we call synchronize_sched()). However, we need to make sure
every critical region have ended before we re-arm a marker that has been
unregistered and then registered back with a different probe data.

Signed-off-by: Mathieu Desnoyers <mathieu.desnoyers@...ymtl.ca>
CC: Christoph Hellwig <hch@...radead.org>
---
 kernel/marker.c |   22 ++++++++++++++++++++--
 1 file changed, 20 insertions(+), 2 deletions(-)

Index: linux-2.6-lttng/kernel/marker.c
===================================================================
--- linux-2.6-lttng.orig/kernel/marker.c	2007-07-14 19:38:11.000000000 -0400
+++ linux-2.6-lttng/kernel/marker.c	2007-07-14 19:46:25.000000000 -0400
@@ -30,11 +30,21 @@ extern struct __mark_marker __stop___mar
 
 /*
  * module_mutex nests inside markers_mutex. Markers mutex protects the builtin
  * and module markers, and the hash table.
  */
 DEFINE_MUTEX(markers_mutex);
 
 /*
+ * Marker deferred synchronization.
+ * Upon marker probe_unregister, we delay call to synchronize_sched() to
+ * accelerate mass unregistration (only when there is no more reference to a
+ * give module do we call synchronize_sched()). However, we need to make sure
+ * every critical region have ended before we re-arm a marker that has been
+ * unregistered and then registered back with a different probe data.
+ */
+static int deferred_sync;
+
+/*
  * Marker hash table, containing the active markers.
  * Protected by module_mutex.
  */
@@ -303,8 +313,10 @@ static inline void __marker_update_probe
 			__stop___markers, probe_module, &refcount);
 	/* Markers in modules. */
 	__marker_update_probes_modules(probe_module, &refcount);
-	if (probe_module && refcount == 0)
+	if (probe_module && refcount == 0) {
 		synchronize_sched();
+		deferred_sync = 0;
+	}
 }
 
 #ifdef CONFIG_MODULES
@@ -353,6 +365,10 @@ int marker_probe_register(const char *na
 		ret = -EBUSY;
 		goto end;
 	}
+	if (deferred_sync) {
+		synchronize_sched();
+		deferred_sync = 0;
+	}
 	ret = _add_marker(name, format, probe, pdata);
 	if (ret)
 		goto end;
@@ -384,6 +400,7 @@ void *marker_probe_unregister(const char
 	/* In what module is the probe handler ? */
 	probe_module = __module_text_address((unsigned long)entry->probe);
 	pdata = _remove_marker(name);
+	deferred_sync = 1;
 	_marker_update_probes(probe_module);
 end:
 	mutex_unlock(&markers_mutex);
@@ -425,6 +442,7 @@ iter_end:
 	/* In what module is the probe handler ? */
 	probe_module = __module_text_address((unsigned long)entry->probe);
 	pdata = _remove_marker(entry->name);
+	deferred_sync = 1;
 	_marker_update_probes(probe_module);
 end:
 	mutex_unlock(&markers_mutex);
@@ -30,11 +30,21 @@ extern struct __mark_marker __stop___mar
 /*
  * module_mutex nests inside markers_mutex. Markers mutex protects the builtin
- * and module markers, and the hash table.
+ * and module markers, the hash table and deferred_sync.
  */
 DEFINE_MUTEX(markers_mutex);
-- 
Mathieu Desnoyers
Computer Engineering Ph.D. Student, Ecole Polytechnique de Montreal
OpenPGP key fingerprint: 8CD5 52C3 8E3C 4140 715F  BA06 3F25 A8FE 3BAE 9A68
-
To unsubscribe from this list: send the line "unsubscribe linux-kernel" in
the body of a message to majordomo@...r.kernel.org
More majordomo info at  http://vger.kernel.org/majordomo-info.html
Please read the FAQ at  http://www.tux.org/lkml/

Powered by blists - more mailing lists

Powered by Openwall GNU/*/Linux Powered by OpenVZ